from functools import partial
from typing import Callable

import torch


def custom_amp_decorator(dec: Callable, cuda_amp_deprecated: bool):
    def decorator(*args, **kwargs):
        if cuda_amp_deprecated:
            kwargs["device_type"] = "cuda"
        return dec(*args, **kwargs)

    return decorator


if hasattr(torch.amp, "custom_fwd"):  # type: ignore[attr-defined]
    deprecated = True
    from torch.amp import custom_bwd, custom_fwd  # type: ignore[attr-defined]
else:
    deprecated = False
    from torch.cuda.amp import custom_bwd, custom_fwd

custom_fwd = custom_amp_decorator(custom_fwd, deprecated)
custom_bwd = custom_amp_decorator(custom_bwd, deprecated)



try:
    from mamba_ssm._version import version as __version__
except ImportError:
    __version__ = "0.0.0"

import platform
import sys

if sys.platform == "darwin" and platform.machine() == "arm64":
    print("Mamba SSM macOS: Running on Apple Silicon with MPS acceleration")

from mamba_ssm.models.mixer_seq_simple import MambaLMHeadModel
from mamba_ssm.modules.mamba2 import Mamba2
from mamba_ssm.modules.mamba_simple import Mamba
from mamba_ssm.ops.selective_scan_interface import mamba_inner_fn, selective_scan_fn
from mamba_ssm.utils.macos import (
    create_tokenizer,
    generate_text_with_model,
    get_device,
    load_and_prepare_model,
)

__all__ = [
    "selective_scan_fn",
    "mamba_inner_fn",
    "Mamba",
    "Mamba2",
    "MambaLMHeadModel",
    "get_device",
    "create_tokenizer",
    "generate_text_with_model",
    "load_and_prepare_model",
]



import unittest

import torch
import torch.nn as nn
import torch.nn.functional as F
from einops import rearrange

from mamba_ssm.modules.mamba_simple import Mamba
from mamba_ssm.utils.generation import InferenceParams


class SimpleMambaLMHeadModel(nn.Module):
    """A simple language model using Mamba for generation testing on macOS."""

    def __init__(
        self,
        d_model=128,
        n_layer=2,
        vocab_size=1000,
        d_state=16,
        device="cpu",
        dtype=torch.float32,
    ):
        super().__init__()
        self.embedding = nn.Embedding(vocab_size, d_model)

        # Create a stack of Mamba layers (no Mamba2 which requires Triton)
        self.layers = nn.ModuleList(
            [Mamba(d_model=d_model, d_state=d_state, d_conv=4, expand=2) for _ in range(n_layer)]
        )

        self.norm = nn.LayerNorm(d_model)
        self.lm_head = nn.Linear(d_model, vocab_size, bias=False)

        # Move model to specified device and dtype
        self.to(device=device, dtype=dtype)

    def forward(
        self,
        input_ids,
        inference_params=None,
        seq_idx=None,
        cu_seqlens=None,
        num_last_tokens=None,
    ):
        """Forward pass with optional inference parameters for generation."""
        x = self.embedding(input_ids)

        for layer in self.layers:
            x = layer(x)

        x = self.norm(x)
        logits = self.lm_head(x)

        return type("", (), {"logits": logits})()  # Return object with logits attribute

    def generate(
        self,
        input_ids,
        max_length,
        output_scores=False,
        return_dict_in_generate=False,
        cg=False,
        teacher_outputs=None,
    ):
        """Simple generation function for testing."""
        batch_size = input_ids.shape[0]
        device = input_ids.device
        generated_tokens = input_ids.clone()
        scores = []

        # Generate one token at a time until we reach max_length
        for _ in range(input_ids.shape[1], max_length):
            outputs = self.forward(generated_tokens)
            next_token_logits = outputs.logits[:, -1, :]

            # For testing, use teacher_outputs if provided, otherwise sample
            if teacher_outputs is not None:
                next_token = teacher_outputs[:, generated_tokens.shape[1]]
            else:
                # Simple greedy decoding
                next_token = next_token_logits.argmax(dim=-1)

            scores.append(next_token_logits)
            generated_tokens = torch.cat([generated_tokens, next_token.unsqueeze(-1)], dim=-1)

        if return_dict_in_generate:
            return type("", (), {"sequences": generated_tokens, "scores": scores})()
        return generated_tokens


class TestGenerationMacOS(unittest.TestCase):
    """Tests for Mamba generation on macOS Apple Silicon."""

    def test_generation(self):
        """Test basic generation functionality on macOS."""
        batch, seqlen, device, dtype = 2, 10, "cpu", torch.float32

        torch.manual_seed(2357)
        model = SimpleMambaLMHeadModel(
            d_model=128,
            n_layer=2,
            vocab_size=1000,
            d_state=16,
            device=device,
            dtype=dtype,
        )

        x = torch.randint(0, 1000, (batch, seqlen), device=device, dtype=torch.long)
        out_ref = model(x).logits
        prompt_len = seqlen // 2
        out = model.generate(
            input_ids=x[:, :prompt_len],
            max_length=seqlen,
            output_scores=True,
            return_dict_in_generate=True,
            teacher_outputs=x,
        )
        out_scores = torch.stack(out.scores, dim=1)
        max_diff = (out_scores - out_ref[:, prompt_len - 1 : -1]).abs().max().item()
        print(f"Max diff: {max_diff}")
        self.assertTrue(
            torch.allclose(out_scores, out_ref[:, prompt_len - 1 : -1], rtol=1e-3, atol=1e-2)
        )

    def test_generation_varlen(self):
        """Test variable length generation on macOS."""
        # For macOS testing, let's simplify this test to just test multiple sequences
        # without the full varlen machinery that might depend on CUDA optimizations
        seqlens = [8, 6, 7]
        batch_size = len(seqlens)
        vocab_size, max_seqlen, device, dtype = 1000, max(seqlens), "cpu", torch.float32

        # Create our simpler model
        torch.manual_seed(2357)
        model = SimpleMambaLMHeadModel(
            d_model=64,
            n_layer=2,
            vocab_size=vocab_size,
            d_state=16,
            device=device,
            dtype=dtype,
        )

        # Create input sequences of different lengths
        input_ids = torch.zeros((batch_size, max_seqlen), device=device, dtype=torch.long)
        for i, seqlen in enumerate(seqlens):
            input_ids[i, :seqlen] = torch.randint(0, vocab_size, (seqlen,), device=device)

        # Create attention mask to handle variable lengths
        attention_mask = torch.zeros_like(input_ids)
        for i, seqlen in enumerate(seqlens):
            attention_mask[i, :seqlen] = 1

        # Forward pass
        out_ref = model(input_ids).logits

        # Test generation with different prompt lengths
        prompt_lens = [seqlen // 2 for seqlen in seqlens]

        # Generate individually
        individual_outputs = []
        for i in range(batch_size):
            # Use only the prompt portion
            prompt = input_ids[i : i + 1, : prompt_lens[i]]
            # Generate up to the original sequence length
            out = model.generate(
                input_ids=prompt,
                max_length=seqlens[i],
                output_scores=True,
                return_dict_in_generate=True,
                teacher_outputs=input_ids[i : i + 1],
            )
            individual_outputs.append(out.sequences)

        # Check that generation works for each sequence
        for i, output in enumerate(individual_outputs):
            # Verify the shape of the output
            self.assertEqual(output.shape, (1, seqlens[i]))
            # Verify the prompt portion matches
            self.assertTrue(
                torch.all(output[0, : prompt_lens[i]] == input_ids[i, : prompt_lens[i]])
            )

        print("Variable length generation test passed")


def run_generation_tests():
    print("🧪 Running Generation macOS tests...")

    try:
        suite = unittest.TestLoader().loadTestsFromTestCase(TestGenerationMacOS)
        runner = unittest.TextTestRunner(verbosity=2)
        result = runner.run(suite)

        if result.wasSuccessful():
            print("✅ All Generation macOS tests passed")
            return True
        else:
            print(f"❌ {len(result.failures)} failures, {len(result.errors)} errors")
            return False

    except Exception as e:
        print(f"❌ Test execution error: {e}")
        return False


if __name__ == "__main__":
    run_generation_tests()



import unittest

import torch
import torch.nn as nn

import mamba_ssm
from mamba_ssm.modules.mamba_simple import Mamba


class MambaClassifier(nn.Module):
    """A simple classifier using Mamba for sequence modeling."""

    def __init__(self, vocab_size=1000, d_model=64, d_state=16, n_classes=10):
        super().__init__()
        self.embedding = nn.Embedding(vocab_size, d_model)
        self.mamba = Mamba(d_model=d_model, d_state=d_state, d_conv=4, expand=2)
        self.classifier = nn.Linear(d_model, n_classes)

    def forward(self, x):
        # x shape: [batch_size, seq_len]
        x = self.embedding(x)  # [batch_size, seq_len, d_model]
        x = self.mamba(x)  # [batch_size, seq_len, d_model]

        # Use last token for classification
        x = x[:, -1]  # [batch_size, d_model]
        x = self.classifier(x)  # [batch_size, n_classes]
        return x


class TestMambaMacOS(unittest.TestCase):
    """Tests for Mamba running on macOS Apple Silicon."""

    def test_basic_forward(self):
        """Test a basic Mamba model forward pass."""
        # Create a small model for testing
        d_model = 16
        d_state = 4
        batch_size = 2
        seq_len = 6

        # Initialize model
        model = Mamba(d_model=d_model, d_state=d_state, d_conv=2, expand=2)

        # Create input
        x = torch.randn(batch_size, seq_len, d_model)

        # Forward pass
        out = model(x)

        # Check output shape
        self.assertEqual(out.shape, (batch_size, seq_len, d_model))

    def test_classifier_forward_backward(self):
        """Test a more complex model with embedding, classification, and backprop."""
        # Model parameters
        vocab_size = 1000
        d_model = 64
        d_state = 16
        n_classes = 10
        batch_size = 8
        seq_len = 32

        # Initialize model
        model = MambaClassifier(
            vocab_size=vocab_size, d_model=d_model, d_state=d_state, n_classes=n_classes
        )

        # Create dummy input (token IDs)
        x = torch.randint(0, vocab_size, (batch_size, seq_len))

        # Forward pass
        out = model(x)

        # Check output shape
        self.assertEqual(out.shape, (batch_size, n_classes))

        # Test backpropagation
        target = torch.randint(0, n_classes, (batch_size,))
        loss_fn = nn.CrossEntropyLoss()
        loss = loss_fn(out, target)

        # This should not raise an error
        loss.backward()


def run_mamba_tests():
    print("🧪 Running Mamba macOS tests...")

    try:
        suite = unittest.TestLoader().loadTestsFromTestCase(TestMambaMacOS)
        runner = unittest.TextTestRunner(verbosity=2)
        result = runner.run(suite)

        if result.wasSuccessful():
            print("✅ All Mamba macOS tests passed")
            return True
        else:
            print(f"❌ {len(result.failures)} failures, {len(result.errors)} errors")
            return False

    except Exception as e:
        print(f"❌ Test execution error: {e}")
        return False


if __name__ == "__main__":
    run_mamba_tests()



import torch
import torch.nn.functional as F
from einops import rearrange, repeat

from mamba_ssm.utils.torch import custom_bwd, custom_fwd

try:
    import causal_conv1d_cuda
    from causal_conv1d import causal_conv1d_fn
except ImportError:
    causal_conv1d_fn = None
    causal_conv1d_cuda = None

try:
    import selective_scan_cuda

    has_cuda_support = True
except ImportError:
    has_cuda_support = False


def _layer_norm_fwd(x, weight, bias, eps, residual=None, residual_dtype=None, is_rms_norm=False):
    if is_rms_norm:
        rstd = 1 / torch.sqrt((x.square()).mean(dim=-1, keepdim=True) + eps)
        out = (x * rstd * weight) + (bias if bias is not None else 0)
        return (out, None)
    return (F.layer_norm(x, x.shape[-1:], weight=weight, bias=bias, eps=eps), None)


class SelectiveScanFn(torch.autograd.Function):

    @staticmethod
    def forward(
        ctx,
        u,
        delta,
        A,
        B,
        C,
        D=None,
        z=None,
        delta_bias=None,
        delta_softplus=False,
        return_last_state=False,
    ):
        if not has_cuda_support:
            # Fall back to reference implementation
            out = selective_scan_ref(
                u, delta, A, B, C, D, z, delta_bias, delta_softplus, return_last_state
            )
            if return_last_state:
                return out
            else:
                # Store None values in context since we won't use them for backward
                ctx.save_for_backward(u, delta, A, B, C, D, None if z is None else z, delta_bias)
                ctx.has_z = z is not None
                ctx.delta_softplus = delta_softplus
                return out

        if u.stride(-1) != 1:
            u = u.contiguous()
        if delta.stride(-1) != 1:
            delta = delta.contiguous()
        if D is not None:
            D = D.contiguous()
        if B.stride(-1) != 1:
            B = B.contiguous()
        if C.stride(-1) != 1:
            C = C.contiguous()
        if z is not None and z.stride(-1) != 1:
            z = z.contiguous()
        if B.dim() == 3:
            B = rearrange(B, "b dstate l -> b 1 dstate l")
            ctx.squeeze_B = True
        if C.dim() == 3:
            C = rearrange(C, "b dstate l -> b 1 dstate l")
            ctx.squeeze_C = True
        out, x, *rest = selective_scan_cuda.fwd(u, delta, A, B, C, D, z, delta_bias, delta_softplus)
        ctx.delta_softplus = delta_softplus
        ctx.has_z = z is not None
        last_state = x[:, :, -1, 1::2]  # (batch, dim, dstate)
        if not ctx.has_z:
            ctx.save_for_backward(u, delta, A, B, C, D, delta_bias, x)
            return out if not return_last_state else (out, last_state)
        else:
            ctx.save_for_backward(u, delta, A, B, C, D, z, delta_bias, x, out)
            out_z = rest[0]
            return out_z if not return_last_state else (out_z, last_state)

    @staticmethod
    def backward(ctx, dout, *args):
        if not has_cuda_support:
            # Use autograd to compute gradients through the reference implementation
            u, delta, A, B, C, D, z, delta_bias = ctx.saved_tensors

            # Set requires_grad on input tensors
            u.requires_grad_(True)
            delta.requires_grad_(True)
            A.requires_grad_(True)
            B.requires_grad_(True)
            C.requires_grad_(True)
            if D is not None:
                D.requires_grad_(True)
            if z is not None:
                z.requires_grad_(True)
            if delta_bias is not None:
                delta_bias.requires_grad_(True)

            # Forward pass with gradients enabled
            with torch.enable_grad():
                out = selective_scan_ref(
                    u,
                    delta,
                    A,
                    B,
                    C,
                    D,
                    z,
                    delta_bias,
                    ctx.delta_softplus,
                    return_last_state=False,
                )

            # Compute gradients using torch.autograd.grad
            inputs_to_grad = [u, delta, A, B, C]
            if D is not None:
                inputs_to_grad.append(D)
            if z is not None:
                inputs_to_grad.append(z)
            if delta_bias is not None:
                inputs_to_grad.append(delta_bias)

            grads = torch.autograd.grad(
                outputs=out,
                inputs=inputs_to_grad,
                grad_outputs=dout,
                allow_unused=True,
                create_graph=False,
            )

            du = grads[0]
            ddelta = grads[1]
            dA = grads[2]
            dB = grads[3]
            dC = grads[4]
            idx = 5
            dD = grads[idx] if D is not None else None
            if D is not None:
                idx += 1
            dz = grads[idx] if z is not None else None
            if z is not None:
                idx += 1
            ddelta_bias = grads[idx] if delta_bias is not None else None

            return (du, ddelta, dA, dB, dC, dD, dz, ddelta_bias, None, None)

        if not ctx.has_z:
            u, delta, A, B, C, D, delta_bias, x = ctx.saved_tensors
            z = None
            out = None
        else:
            u, delta, A, B, C, D, z, delta_bias, x, out = ctx.saved_tensors
        if dout.stride(-1) != 1:
            dout = dout.contiguous()
        # The kernel supports passing in a pre-allocated dz (e.g., in case we want to fuse the
        # backward of selective_scan_cuda with the backward of chunk).
        # Here we just pass in None and dz will be allocated in the C++ code.
        du, ddelta, dA, dB, dC, dD, ddelta_bias, *rest = selective_scan_cuda.bwd(
            u,
            delta,
            A,
            B,
            C,
            D,
            z,
            delta_bias,
            dout,
            x,
            out,
            None,
            ctx.delta_softplus,
            False,  # option to recompute out_z, not used here
        )
        dz = rest[0] if ctx.has_z else None
        dB = dB.squeeze(1) if getattr(ctx, "squeeze_B", False) else dB
        dC = dC.squeeze(1) if getattr(ctx, "squeeze_C", False) else dC
        return (
            du,
            ddelta,
            dA,
            dB,
            dC,
            dD if D is not None else None,
            dz,
            ddelta_bias if delta_bias is not None else None,
            None,
            None,
        )


def rms_norm_forward(
    x,
    weight,
    bias,
    eps=1e-6,
    is_rms_norm=True,
):
    # x (b l) d
    if x.stride(-1) != 1:
        x = x.contiguous()
    weight = weight.contiguous()
    if bias is not None:
        bias = bias.contiguous()
    y = _layer_norm_fwd(x, weight, bias, eps, None, residual_dtype=None, is_rms_norm=is_rms_norm)[0]
    # y (b l) d
    return y


def selective_scan_fn(
    u,
    delta,
    A,
    B,
    C,
    D=None,
    z=None,
    delta_bias=None,
    delta_softplus=False,
    return_last_state=False,
):
    """if return_last_state is True, returns (out, last_state)
    last_state has shape (batch, dim, dstate). Note that the gradient of the last state is
    not considered in the backward pass.
    """
    if not has_cuda_support:
        # Use reference implementation directly so PyTorch can auto-differentiate
        return selective_scan_ref(
            u, delta, A, B, C, D, z, delta_bias, delta_softplus, return_last_state
        )
    return SelectiveScanFn.apply(
        u, delta, A, B, C, D, z, delta_bias, delta_softplus, return_last_state
    )


def selective_scan_ref(
    u,
    delta,
    A,
    B,
    C,
    D=None,
    z=None,
    delta_bias=None,
    delta_softplus=False,
    return_last_state=False,
):
    """
    u: r(B D L)
    delta: r(B D L)
    A: c(D N) or r(D N)
    B: c(D N) or r(B N L) or r(B N 2L) or r(B G N L) or (B G N L)
    C: c(D N) or r(B N L) or r(B N 2L) or r(B G N L) or (B G N L)
    D: r(D)
    z: r(B D L)
    delta_bias: r(D), fp32

    out: r(B D L)
    last_state (optional): r(B D dstate) or c(B D dstate)
    """
    dtype_in = u.dtype
    u = u.float()
    delta = delta.float()
    if delta_bias is not None:
        delta = delta + delta_bias[..., None].float()
    if delta_softplus:
        delta = F.softplus(delta)
    batch, dim, dstate = u.shape[0], A.shape[0], A.shape[1]
    is_variable_B = B.dim() >= 3
    is_variable_C = C.dim() >= 3
    if A.is_complex():
        if is_variable_B:
            B = torch.view_as_complex(rearrange(B.float(), "... (L two) -> ... L two", two=2))
        if is_variable_C:
            C = torch.view_as_complex(rearrange(C.float(), "... (L two) -> ... L two", two=2))
    else:
        B = B.float()
        C = C.float()
    x = A.new_zeros((batch, dim, dstate))
    ys = []
    deltaA = torch.exp(torch.einsum("bdl,dn->bdln", delta, A))
    if not is_variable_B:
        deltaB_u = torch.einsum("bdl,dn,bdl->bdln", delta, B, u)
    else:
        if B.dim() == 3:
            deltaB_u = torch.einsum("bdl,bnl,bdl->bdln", delta, B, u)
        else:
            B = repeat(B, "B G N L -> B (G H) N L", H=dim // B.shape[1])
            deltaB_u = torch.einsum("bdl,bdnl,bdl->bdln", delta, B, u)
    if is_variable_C and C.dim() == 4:
        C = repeat(C, "B G N L -> B (G H) N L", H=dim // C.shape[1])
    last_state = None
    for i in range(u.shape[2]):
        x = deltaA[:, :, i] * x + deltaB_u[:, :, i]
        if not is_variable_C:
            y = torch.einsum("bdn,dn->bd", x, C)
        else:
            if C.dim() == 3:
                y = torch.einsum("bdn,bn->bd", x, C[:, :, i])
            else:
                y = torch.einsum("bdn,bdn->bd", x, C[:, :, :, i])
        if i == u.shape[2] - 1:
            last_state = x
        if y.is_complex():
            y = y.real * 2
        ys.append(y)
    y = torch.stack(ys, dim=2)  # (batch dim L)
    out = y if D is None else y + u * rearrange(D, "d -> d 1")
    if z is not None:
        out = out * F.silu(z)
    out = out.to(dtype=dtype_in)
    return out if not return_last_state else (out, last_state)


class MambaInnerFn(torch.autograd.Function):

    @staticmethod
    @custom_fwd
    def forward(
        ctx,
        xz,
        conv1d_weight,
        conv1d_bias,
        x_proj_weight,
        delta_proj_weight,
        out_proj_weight,
        out_proj_bias,
        A,
        B=None,
        C=None,
        D=None,
        delta_bias=None,
        B_proj_bias=None,
        C_proj_bias=None,
        delta_softplus=True,
        checkpoint_lvl=1,
        b_rms_weight=None,
        c_rms_weight=None,
        dt_rms_weight=None,
        b_c_dt_rms_eps=1e-6,
    ):
        """
        xz: (batch, dim, seqlen)
        """
        if causal_conv1d_cuda is None:
            raise ImportError(
                "causal_conv1d_cuda is not available. Please install causal-conv1d or use the reference implementation."
            )
        assert checkpoint_lvl in [0, 1]
        L = xz.shape[-1]
        delta_rank = delta_proj_weight.shape[1]
        d_state = A.shape[-1] * (1 if not A.is_complex() else 2)
        if torch.is_autocast_enabled():
            x_proj_weight = x_proj_weight.to(dtype=torch.get_autocast_gpu_dtype())
            delta_proj_weight = delta_proj_weight.to(dtype=torch.get_autocast_gpu_dtype())
            out_proj_weight = out_proj_weight.to(dtype=torch.get_autocast_gpu_dtype())
            out_proj_bias = (
                out_proj_bias.to(dtype=torch.get_autocast_gpu_dtype())
                if out_proj_bias is not None
                else None
            )
        if xz.stride(-1) != 1:
            xz = xz.contiguous()
        conv1d_weight = rearrange(conv1d_weight, "d 1 w -> d w")
        x, z = xz.chunk(2, dim=1)
        conv1d_bias = conv1d_bias.contiguous() if conv1d_bias is not None else None
        conv1d_out = causal_conv1d_cuda.causal_conv1d_fwd(
            x, conv1d_weight, conv1d_bias, None, None, None, True
        )
        # We're being very careful here about the layout, to avoid extra transposes.
        # We want delta to have d as the slowest moving dimension
        # and L as the fastest moving dimension, since those are what the ssm_scan kernel expects.
        x_dbl = F.linear(rearrange(conv1d_out, "b d l -> (b l) d"), x_proj_weight)  # (bl d)
        delta = rearrange(delta_proj_weight @ x_dbl[:, :delta_rank].t(), "d (b l) -> b d l", l=L)
        ctx.is_variable_B = B is None
        ctx.is_variable_C = C is None
        ctx.B_proj_bias_is_None = B_proj_bias is None
        ctx.C_proj_bias_is_None = C_proj_bias is None
        if B is None:  # variable B
            B = x_dbl[:, delta_rank : delta_rank + d_state]  # (bl dstate)
            if B_proj_bias is not None:
                B = B + B_proj_bias.to(dtype=B.dtype)
            if not A.is_complex():
                # B = rearrange(B, "(b l) dstate -> b dstate l", l=L).contiguous()
                B = rearrange(B, "(b l) dstate -> b 1 dstate l", l=L).contiguous()
            else:
                B = rearrange(
                    B, "(b l) (dstate two) -> b 1 dstate (l two)", l=L, two=2
                ).contiguous()
        else:
            if B.stride(-1) != 1:
                B = B.contiguous()
        if C is None:  # variable C
            C = x_dbl[:, -d_state:]  # (bl dstate)
            if C_proj_bias is not None:
                C = C + C_proj_bias.to(dtype=C.dtype)
            if not A.is_complex():
                # C = rearrange(C, "(b l) dstate -> b dstate l", l=L).contiguous()
                C = rearrange(C, "(b l) dstate -> b 1 dstate l", l=L).contiguous()
            else:
                C = rearrange(
                    C, "(b l) (dstate two) -> b 1 dstate (l two)", l=L, two=2
                ).contiguous()
        else:
            if C.stride(-1) != 1:
                C = C.contiguous()
        if D is not None:
            D = D.contiguous()

        if b_rms_weight is not None:
            B = rearrange(B, "b 1 dstate l -> (b l) dstate", l=L).contiguous()
            B = rms_norm_forward(B, b_rms_weight, bias=None, eps=b_c_dt_rms_eps)
            B = rearrange(B, "(b l) dstate -> b 1 dstate l", l=L).contiguous()
        if c_rms_weight is not None:
            C = rearrange(C, "b 1 dstate l -> (b l) dstate", l=L).contiguous()
            C = rms_norm_forward(C, c_rms_weight, bias=None, eps=b_c_dt_rms_eps)
            C = rearrange(C, "(b l) dstate -> b 1 dstate l", l=L).contiguous()
        if dt_rms_weight is not None:
            delta = rearrange(delta, "b d l -> (b l) d", l=L).contiguous()
            delta = rms_norm_forward(delta, dt_rms_weight, bias=None, eps=b_c_dt_rms_eps)
            delta = rearrange(delta, "(b l) d -> b d l", l=L).contiguous()

        out, scan_intermediates, out_z = selective_scan_cuda.fwd(
            conv1d_out, delta, A, B, C, D, z, delta_bias, delta_softplus
        )
        ctx.delta_softplus = delta_softplus
        ctx.out_proj_bias_is_None = out_proj_bias is None
        ctx.checkpoint_lvl = checkpoint_lvl
        ctx.b_rms_weight = b_rms_weight
        ctx.c_rms_weight = c_rms_weight
        ctx.dt_rms_weight = dt_rms_weight
        ctx.b_c_dt_rms_eps = b_c_dt_rms_eps
        if checkpoint_lvl >= 1:  # Will recompute conv1d_out and delta in the backward pass
            conv1d_out, delta = None, None
        ctx.save_for_backward(
            xz,
            conv1d_weight,
            conv1d_bias,
            x_dbl,
            x_proj_weight,
            delta_proj_weight,
            out_proj_weight,
            conv1d_out,
            delta,
            A,
            B,
            C,
            D,
            delta_bias,
            scan_intermediates,
            b_rms_weight,
            c_rms_weight,
            dt_rms_weight,
            out,
        )
        return F.linear(rearrange(out_z, "b d l -> b l d"), out_proj_weight, out_proj_bias)

    @staticmethod
    @custom_bwd
    def backward(ctx, dout):
        # dout: (batch, seqlen, dim)
        assert (
            causal_conv1d_cuda is not None
        ), "causal_conv1d_cuda is not available. Please install causal-conv1d."
        (
            xz,
            conv1d_weight,
            conv1d_bias,
            x_dbl,
            x_proj_weight,
            delta_proj_weight,
            out_proj_weight,
            conv1d_out,
            delta,
            A,
            B,
            C,
            D,
            delta_bias,
            scan_intermediates,
            b_rms_weight,
            c_rms_weight,
            dt_rms_weight,
            out,
        ) = ctx.saved_tensors
        L = xz.shape[-1]
        delta_rank = delta_proj_weight.shape[1]
        d_state = A.shape[-1] * (1 if not A.is_complex() else 2)
        x, z = xz.chunk(2, dim=1)
        if dout.stride(-1) != 1:
            dout = dout.contiguous()
        if ctx.checkpoint_lvl == 1:
            conv1d_out = causal_conv1d_cuda.causal_conv1d_fwd(
                x, conv1d_weight, conv1d_bias, None, None, None, True
            )
            delta = rearrange(
                delta_proj_weight @ x_dbl[:, :delta_rank].t(), "d (b l) -> b d l", l=L
            )
            if dt_rms_weight is not None:
                delta = rearrange(delta, "b d l -> (b l) d", l=L).contiguous()
                delta = rms_norm_forward(delta, ctx.dt_rms_weight, None, ctx.b_c_dt_rms_eps)
                delta = rearrange(delta, "(b l) d -> b d l", l=L).contiguous()
            if b_rms_weight is not None:
                # Recompute & RMSNorm B
                B = rearrange(B, "b 1 dstate l -> (b l) dstate", l=L).contiguous()
                B = rms_norm_forward(B, ctx.b_rms_weight, None, ctx.b_c_dt_rms_eps)
                B = rearrange(B, "(b l) dstate -> b 1 dstate l", l=L).contiguous()
            if c_rms_weight is not None:
                # Recompute & RMSNorm C
                C = rearrange(C, "b 1 dstate l -> (b l) dstate", l=L).contiguous()
                C = rms_norm_forward(C, ctx.c_rms_weight, None, ctx.b_c_dt_rms_eps)
                C = rearrange(C, "(b l) dstate -> b 1 dstate l", l=L).contiguous()

        # The kernel supports passing in a pre-allocated dz (e.g., in case we want to fuse the
        # backward of selective_scan_cuda with the backward of chunk).
        dxz = torch.empty_like(xz)  # (batch, dim, seqlen)
        dx, dz = dxz.chunk(2, dim=1)
        dout = rearrange(dout, "b l e -> e (b l)")
        dout_y = rearrange(out_proj_weight.t() @ dout, "d (b l) -> b d l", l=L)
        dconv1d_out, ddelta, dA, dB, dC, dD, ddelta_bias, dz, out_z = selective_scan_cuda.bwd(
            conv1d_out,
            delta,
            A,
            B,
            C,
            D,
            z,
            delta_bias,
            dout_y,
            scan_intermediates,
            out,
            dz,
            ctx.delta_softplus,
            True,  # option to recompute out_z
        )
        dout_proj_weight = torch.einsum("eB,dB->ed", dout, rearrange(out_z, "b d l -> d (b l)"))
        dout_proj_bias = dout.sum(dim=(0, 1)) if not ctx.out_proj_bias_is_None else None
        dD = dD if D is not None else None
        dx_dbl = torch.empty_like(x_dbl)
        dB_proj_bias = None
        if ctx.is_variable_B:
            if not A.is_complex():
                dB = rearrange(dB, "b 1 dstate l -> (b l) dstate").contiguous()
            else:
                dB = rearrange(dB, "b 1 dstate (l two) -> (b l) (dstate two)", two=2).contiguous()
            dB_proj_bias = dB.sum(0) if not ctx.B_proj_bias_is_None else None
            dx_dbl[:, delta_rank : delta_rank + d_state] = dB  # (bl d)
            dB = None
        dC_proj_bias = None
        if ctx.is_variable_C:
            if not A.is_complex():
                dC = rearrange(dC, "b 1 dstate l -> (b l) dstate").contiguous()
            else:
                dC = rearrange(dC, "b 1 dstate (l two) -> (b l) (dstate two)", two=2).contiguous()
            dC_proj_bias = dC.sum(0) if not ctx.C_proj_bias_is_None else None
            dx_dbl[:, -d_state:] = dC  # (bl d)
            dC = None
        ddelta = rearrange(ddelta, "b d l -> d (b l)")
        ddelta_proj_weight = torch.einsum("dB,Br->dr", ddelta, x_dbl[:, :delta_rank])
        dx_dbl[:, :delta_rank] = torch.einsum("dB,dr->Br", ddelta, delta_proj_weight)
        dconv1d_out = rearrange(dconv1d_out, "b d l -> d (b l)")
        dx_proj_weight = torch.einsum(
            "Br,Bd->rd", dx_dbl, rearrange(conv1d_out, "b d l -> (b l) d")
        )
        dconv1d_out = torch.addmm(dconv1d_out, x_proj_weight.t(), dx_dbl.t(), out=dconv1d_out)
        dconv1d_out = rearrange(dconv1d_out, "d (b l) -> b d l", b=x.shape[0], l=x.shape[-1])
        # The kernel supports passing in a pre-allocated dx (e.g., in case we want to fuse the
        # backward of conv1d with the backward of chunk).
        dx, dconv1d_weight, dconv1d_bias, *_ = causal_conv1d_cuda.causal_conv1d_bwd(
            x,
            conv1d_weight,
            conv1d_bias,
            dconv1d_out,
            None,
            None,
            None,
            dx,
            False,
            True,
        )
        dconv1d_bias = dconv1d_bias if conv1d_bias is not None else None
        dconv1d_weight = rearrange(dconv1d_weight, "d w -> d 1 w")
        return (
            dxz,
            dconv1d_weight,
            dconv1d_bias,
            dx_proj_weight,
            ddelta_proj_weight,
            dout_proj_weight,
            dout_proj_bias,
            dA,
            dB,
            dC,
            dD,
            ddelta_bias if delta_bias is not None else None,
            # 6-None are delta_softplus, checkpoint_lvl, b_rms_weight, c_rms_weight, dt_rms_weight, b_c_dt_rms_eps
            dB_proj_bias,
            dC_proj_bias,
            None,
            None,
            None,
            None,
            None,
            None,
        )


def mamba_inner_fn(
    xz,
    conv1d_weight,
    conv1d_bias,
    x_proj_weight,
    delta_proj_weight,
    out_proj_weight,
    out_proj_bias,
    A,
    B=None,
    C=None,
    D=None,
    delta_bias=None,
    B_proj_bias=None,
    C_proj_bias=None,
    delta_softplus=True,
    checkpoint_lvl=1,
    b_rms_weight=None,
    c_rms_weight=None,
    dt_rms_weight=None,
    b_c_dt_rms_eps=1e-6,
):
    if causal_conv1d_cuda is None or not has_cuda_support:
        # Fall back to reference implementation
        return mamba_inner_ref(
            xz,
            conv1d_weight,
            conv1d_bias,
            x_proj_weight,
            delta_proj_weight,
            out_proj_weight,
            out_proj_bias,
            A,
            B,
            C,
            D,
            delta_bias,
            B_proj_bias,
            C_proj_bias,
            delta_softplus,
        )

    return MambaInnerFn.apply(
        xz,
        conv1d_weight,
        conv1d_bias,
        x_proj_weight,
        delta_proj_weight,
        out_proj_weight,
        out_proj_bias,
        A,
        B,
        C,
        D,
        delta_bias,
        B_proj_bias,
        C_proj_bias,
        delta_softplus,
        checkpoint_lvl,
        b_rms_weight,
        c_rms_weight,
        dt_rms_weight,
        b_c_dt_rms_eps,
    )


def mamba_inner_ref(
    xz,
    conv1d_weight,
    conv1d_bias,
    x_proj_weight,
    delta_proj_weight,
    out_proj_weight,
    out_proj_bias,
    A,
    B=None,
    C=None,
    D=None,
    delta_bias=None,
    B_proj_bias=None,
    C_proj_bias=None,
    delta_softplus=True,
):
    assert (
        causal_conv1d_fn is not None
    ), "causal_conv1d_fn is not available. Please install causal-conv1d."
    L = xz.shape[-1]
    delta_rank = delta_proj_weight.shape[1]
    d_state = A.shape[-1] * (1 if not A.is_complex() else 2)
    x, z = xz.chunk(2, dim=1)
    x = causal_conv1d_fn(
        x, rearrange(conv1d_weight, "d 1 w -> d w"), conv1d_bias, activation="silu"
    )
    # We're being very careful here about the layout, to avoid extra transposes.
    # We want delta to have d as the slowest moving dimension
    # and L as the fastest moving dimension, since those are what the ssm_scan kernel expects.
    x_dbl = F.linear(rearrange(x, "b d l -> (b l) d"), x_proj_weight)  # (bl d)
    delta = delta_proj_weight @ x_dbl[:, :delta_rank].t()
    delta = rearrange(delta, "d (b l) -> b d l", l=L)
    if B is None:  # variable B
        B = x_dbl[:, delta_rank : delta_rank + d_state]  # (bl d)
        if B_proj_bias is not None:
            B = B + B_proj_bias.to(dtype=B.dtype)
        if not A.is_complex():
            B = rearrange(B, "(b l) dstate -> b dstate l", l=L).contiguous()
        else:
            B = rearrange(B, "(b l) (dstate two) -> b dstate (l two)", l=L, two=2).contiguous()
    if C is None:  # variable B
        C = x_dbl[:, -d_state:]  # (bl d)
        if C_proj_bias is not None:
            C = C + C_proj_bias.to(dtype=C.dtype)
        if not A.is_complex():
            C = rearrange(C, "(b l) dstate -> b dstate l", l=L).contiguous()
        else:
            C = rearrange(C, "(b l) (dstate two) -> b dstate (l two)", l=L, two=2).contiguous()
    y = selective_scan_fn(x, delta, A, B, C, D, z=z, delta_bias=delta_bias, delta_softplus=True)
    return F.linear(rearrange(y, "b d l -> b l d"), out_proj_weight, out_proj_bias)



import unittest

import torch
import torch.nn as nn

from mamba_ssm.modules.mamba2 import Mamba2


class TestMamba2(unittest.TestCase):
    def test_basic_forward(self):
        d_model, d_state, batch_size, seq_len = 64, 16, 2, 8
        model = Mamba2(d_model=d_model, d_state=d_state, d_conv=4, expand=2)
        x = torch.randn(batch_size, seq_len, d_model)
        out = model(x)
        self.assertEqual(out.shape, (batch_size, seq_len, d_model))

    def test_shape_consistency_large_model(self):
        d_model, d_state, headdim, batch_size, seq_len = 256, 64, 64, 2, 32
        model = Mamba2(d_model=d_model, d_state=d_state, headdim=headdim, d_conv=4, expand=2)
        x = torch.randn(batch_size, seq_len, d_model)
        out = model(x)
        self.assertEqual(out.shape, (batch_size, seq_len, d_model))
        self.assertEqual(model.d_inner, 512)
        self.assertEqual(model.nheads, 8)
        self.assertEqual(model.headdim, 64)

    def test_variable_length(self):
        d_model, d_state, batch_size = 64, 16, 2
        seq_lens = [5, 8]
        model = Mamba2(d_model=d_model, d_state=d_state, d_conv=4, expand=2)
        for seq_len in seq_lens:
            x = torch.randn(batch_size, seq_len, d_model)
            out = model(x)
            self.assertEqual(out.shape, (batch_size, seq_len, d_model))

    def test_different_headdim_configs(self):
        d_model, d_state, batch_size, seq_len = 192, 48, 2, 16
        for headdim in [32, 48, 64]:
            with self.subTest(headdim=headdim):
                model = Mamba2(
                    d_model=d_model, d_state=d_state, headdim=headdim, d_conv=4, expand=2
                )
                expected_nheads = model.d_inner // headdim
                self.assertEqual(model.nheads, expected_nheads)
                x = torch.randn(batch_size, seq_len, d_model)
                out = model(x)
                self.assertEqual(out.shape, (batch_size, seq_len, d_model))

    def test_gradient_flow(self):
        d_model, d_state, batch_size, seq_len = 64, 16, 2, 8
        model = Mamba2(d_model=d_model, d_state=d_state, d_conv=4, expand=2)
        x = torch.randn(batch_size, seq_len, d_model, requires_grad=True)
        out = model(x)
        loss = nn.MSELoss()(out, torch.randn_like(out))
        loss.backward()
        self.assertIsNotNone(x.grad)
        self.assertIsNotNone(model.in_proj.weight.grad)
        self.assertIsNotNone(model.out_proj.weight.grad)
        self.assertIsNotNone(model.conv1d.weight.grad)
        self.assertIsNotNone(model.A_log.grad)
        self.assertIsNotNone(model.D.grad)
        self.assertIsNotNone(model.dt_bias.grad)

    def test_edge_case_dimensions(self):
        test_configs = [
            {"d_model": 32, "d_state": 8, "headdim": 16},
            {"d_model": 512, "d_state": 128, "headdim": 128},
            {"d_model": 96, "d_state": 24, "headdim": 24},
        ]
        for config in test_configs:
            with self.subTest(**config):
                model = Mamba2(
                    d_model=config["d_model"],
                    d_state=config["d_state"],
                    headdim=config["headdim"],
                    d_conv=4,
                    expand=2,
                )
                x = torch.randn(1, 4, config["d_model"])
                out = model(x)
                self.assertEqual(out.shape, (1, 4, config["d_model"]))


def run_mamba2_tests():
    print("🧪 Running Mamba2 tests...")
    try:
        suite = unittest.TestLoader().loadTestsFromTestCase(TestMamba2)
        result = unittest.TextTestRunner(verbosity=2).run(suite)
        if result.wasSuccessful():
            print("✅ All Mamba2 tests passed")
            return True
        print(f"❌ {len(result.failures)} failures, {len(result.errors)} errors")
        return False
    except Exception as e:
        print(f"❌ Test error: {e}")
        return False


if __name__ == "__main__":
    run_mamba2_tests()



#!/usr/bin/env python3
import argparse

from mamba_ssm import generate_text_with_model, get_device, load_and_prepare_model


def run_generation(model, tokenizer, args, model_name):
    generated_text = generate_text_with_model(
        model, tokenizer, args.prompt, args.device, args.max_length, args.temperature
    )
    print(f"\n📝 Generated text:\n'{generated_text}'")


def run_model(model_name, args):
    print(f"🍎 Loading downloaded {model_name.title()} model on {args.device}...")

    try:
        success, model, tokenizer = load_and_prepare_model(model_name, args.model_dir, args.device)
        if not success:
            return False

        print(f"💭 Input prompt: '{args.prompt}'")
        run_generation(model, tokenizer, args, model_name)
        return True

    except Exception as e:
        print(f"❌ Failed: {e}")
        return False


def main():
    parser = argparse.ArgumentParser(description="Run Mamba models")
    parser.add_argument("model", choices=["mamba1", "mamba2"], help="Model to run")
    parser.add_argument("--prompt", default="The future of AI is", help="Text prompt")
    parser.add_argument("--max-length", type=int, default=50, help="Maximum generation length")
    parser.add_argument("--temperature", type=float, default=0.8, help="Sampling temperature")
    parser.add_argument("--device", default=get_device(), help="Device")
    parser.add_argument("--model-dir", default="./models", help="Downloaded models directory")

    args = parser.parse_args()
    if args.model == "mamba2" and args.max_length == 50:
        args.max_length = 8

    success = run_model(args.model, args)
    print("✅ SUCCESS" if success else "❌ FAILED")


if __name__ == "__main__":
    main()



import math

import torch
import torch.nn as nn
import torch.nn.functional as F
from einops import rearrange, repeat

try:
    from causal_conv1d import causal_conv1d_fn
except ImportError:
    causal_conv1d_fn = None

try:
    from mamba_ssm.ops.triton.layernorm_gated import RMSNorm as RMSNormGated
except ImportError:

    class RMSNormGated(nn.Module):
        def __init__(self, d, eps=1e-5, norm_before_gate=False, device=None, dtype=None):
            super().__init__()
            self.eps = eps
            self.norm_before_gate = norm_before_gate
            self.weight = nn.Parameter(torch.ones(d, device=device, dtype=dtype))

        def forward(self, x, z=None):
            if z is not None and not self.norm_before_gate:
                x = x * F.silu(z)
            x_norm = x * torch.rsqrt(x.pow(2).mean(-1, keepdim=True) + self.eps)
            x_norm = x_norm * self.weight
            if z is not None and self.norm_before_gate:
                x_norm = x_norm * F.silu(z)
            return x_norm


try:
    from mamba_ssm.ops.triton.ssd_combined import (
        mamba_chunk_scan_combined,
        mamba_split_conv1d_scan_combined,
    )

    USE_OPTIMIZED_KERNELS = True
except ImportError:
    USE_OPTIMIZED_KERNELS = False

    def mamba_chunk_scan_combined(
        x,
        dt,
        A,
        B,
        C,
        chunk_size=256,
        D=None,
        z=None,
        seq_idx=None,
        initial_states=None,
        **kwargs,
    ):
        batch, seqlen, nheads, headdim = x.shape
        ngroups = B.shape[2] if B.ndim > 3 else 1

        if ngroups < nheads:
            heads_per_group = nheads // ngroups
            B = repeat(B, "b l g d -> b l (g h) d", h=heads_per_group)
            C = repeat(C, "b l g d -> b l (g h) d", h=heads_per_group)

        dA = torch.einsum("blh,h->blh", dt, A)
        dB_u = torch.einsum("blh,blhp,blhn->blhpn", dt, x, B)

        dA_padded = F.pad(dA[:, 1:], (0, 0, 0, 1))
        dA_cumsum = dA_padded.flip(1).cumsum(1).flip(1)
        dA_cumsum_exp = torch.exp(dA_cumsum)

        dA_cumsum_expanded = dA_cumsum_exp.unsqueeze(-1).unsqueeze(-1)
        x_scaled = dB_u * dA_cumsum_expanded
        x_cumsum = x_scaled.cumsum(1)
        states = x_cumsum / (dA_cumsum_expanded + 1e-12)

        y = torch.einsum("blhpn,blhn->blhp", states, C)

        if D is not None:
            y = y + x * D.unsqueeze(0).unsqueeze(0).unsqueeze(-1)

        return y

    def mamba_split_conv1d_scan_combined(*args, **kwargs):
        raise NotImplementedError("Reference implementation not available for split conv1d scan")


class Mamba2(nn.Module):
    def __init__(
        self,
        d_model,
        d_state=64,
        d_conv=4,
        conv_init=None,
        expand=2,
        headdim=64,
        ngroups=1,
        A_init_range=(1, 16),
        dt_min=0.001,
        dt_max=0.1,
        dt_init_floor=1e-4,
        dt_limit=(0.0, float("inf")),
        learnable_init_states=False,
        activation="swish",
        bias=False,
        conv_bias=True,
        chunk_size=256,
        use_mem_eff_path=False,
        layer_idx=None,
        device=None,
        dtype=None,
    ):
        factory_kwargs = {"device": device, "dtype": dtype}
        super().__init__()

        self.d_model, self.d_state, self.d_conv, self.conv_init, self.expand = (
            d_model,
            d_state,
            d_conv,
            conv_init,
            expand,
        )
        self.d_inner, self.headdim, self.ngroups = expand * d_model, headdim, ngroups

        assert self.d_inner % self.headdim == 0
        self.nheads = self.d_inner // self.headdim

        self.dt_limit, self.learnable_init_states, self.activation, self.chunk_size = (
            dt_limit,
            learnable_init_states,
            activation,
            chunk_size,
        )
        self.use_mem_eff_path, self.layer_idx = (
            use_mem_eff_path and USE_OPTIMIZED_KERNELS,
            layer_idx,
        )

        d_in_proj = 2 * self.d_inner + 2 * self.ngroups * self.d_state + self.nheads
        self.in_proj = nn.Linear(self.d_model, d_in_proj, bias=bias, **factory_kwargs)

        conv_dim = self.d_inner + 2 * self.ngroups * self.d_state
        self.conv1d = nn.Conv1d(
            conv_dim,
            conv_dim,
            d_conv,
            groups=conv_dim,
            padding=d_conv - 1,
            bias=conv_bias,
            **factory_kwargs,
        )

        if self.conv_init is not None:
            nn.init.uniform_(self.conv1d.weight, -self.conv_init, self.conv_init)

        if self.learnable_init_states:
            self.init_states = nn.Parameter(
                torch.zeros(self.nheads, self.headdim, self.d_state, **factory_kwargs)
            )
            self.init_states._no_weight_decay = True

        self.act = nn.SiLU()

        dt = torch.exp(
            torch.rand(self.nheads, **factory_kwargs) * (math.log(dt_max) - math.log(dt_min))
            + math.log(dt_min)
        )
        dt = torch.clamp(dt, min=dt_init_floor)
        inv_dt = dt + torch.log(-torch.expm1(-dt))
        self.dt_bias = nn.Parameter(inv_dt)
        self.dt_bias._no_weight_decay = True

        A = torch.empty(self.nheads, dtype=torch.float32, device=device).uniform_(*A_init_range)
        self.A_log = nn.Parameter(torch.log(A).to(dtype=dtype))
        self.A_log._no_weight_decay = True

        self.D = nn.Parameter(torch.ones(self.nheads, device=device))
        self.D._no_weight_decay = True

        self.norm = RMSNormGated(self.d_inner, eps=1e-5, norm_before_gate=False, **factory_kwargs)
        self.out_proj = nn.Linear(self.d_inner, self.d_model, bias=bias, **factory_kwargs)

    def forward(self, u, seq_idx=None, inference_params=None, **kwargs):
        batch, seqlen, dim = u.shape
        zxbcdt = self.in_proj(u)
        A = -torch.exp(self.A_log)
        initial_states = (
            repeat(self.init_states, "... -> b ...", b=batch)
            if self.learnable_init_states
            else None
        )
        dt_limit_kwargs = (
            {} if self.dt_limit == (0.0, float("inf")) else dict(dt_limit=self.dt_limit)
        )

        if self.use_mem_eff_path:
            try:
                return mamba_split_conv1d_scan_combined(
                    zxbcdt,
                    rearrange(self.conv1d.weight, "d 1 w -> d w"),
                    self.conv1d.bias,
                    self.dt_bias,
                    A,
                    D=self.D,
                    chunk_size=self.chunk_size,
                    seq_idx=seq_idx,
                    activation=self.activation,
                    rmsnorm_weight=self.norm.weight,
                    rmsnorm_eps=self.norm.eps,
                    outproj_weight=self.out_proj.weight,
                    outproj_bias=self.out_proj.bias,
                    headdim=self.headdim,
                    ngroups=self.ngroups,
                    norm_before_gate=False,
                    initial_states=initial_states,
                    **dt_limit_kwargs,
                )
            except Exception:
                pass

        z, xBC, dt = torch.split(
            zxbcdt,
            [self.d_inner, self.d_inner + 2 * self.ngroups * self.d_state, self.nheads],
            dim=-1,
        )
        dt = F.softplus(dt + self.dt_bias)

        if causal_conv1d_fn is None or self.activation not in ["silu", "swish"]:
            xBC = self.act(self.conv1d(xBC.transpose(1, 2)).transpose(1, 2))[:, :seqlen, :]
        else:
            xBC = causal_conv1d_fn(
                xBC.transpose(1, 2),
                rearrange(self.conv1d.weight, "d 1 w -> d w"),
                self.conv1d.bias,
                self.activation,
            ).transpose(1, 2)

        x, B, C = torch.split(
            xBC,
            [self.d_inner, self.ngroups * self.d_state, self.ngroups * self.d_state],
            dim=-1,
        )

        y = mamba_chunk_scan_combined(
            rearrange(x, "b l (h p) -> b l h p", p=self.headdim),
            dt,
            A,
            rearrange(B, "b l (g n) -> b l g n", g=self.ngroups),
            rearrange(C, "b l (g n) -> b l g n", g=self.ngroups),
            chunk_size=self.chunk_size,
            D=self.D,
            z=None,
            seq_idx=seq_idx,
            initial_states=initial_states,
            **dt_limit_kwargs,
        )

        y = rearrange(y, "b l h p -> b l (h p)")
        y = self.norm(y, z)
        return self.out_proj(y)



"""
Unit tests for selective scan gradient flow verification.
These tests ensure that gradients computed via different methods
are consistent and correctly propagated through the selective scan operation.
"""

import unittest

import torch

from mamba_ssm.ops.selective_scan_interface import selective_scan_fn


class TestSelectiveScanGradients(unittest.TestCase):
    """Test gradient flow through selective scan operations."""

    def test_gradient_matches_explicit_computation(self):
        """Verify backward() and torch.autograd.grad() produce identical gradients."""
        batch, dim, dstate, seq_len = 2, 4, 8, 6
        torch.manual_seed(42)

        u_base = torch.randn(batch, dim, seq_len)
        delta_base = torch.randn(batch, dim, seq_len)
        A_base = torch.randn(dim, dstate)
        B_base = torch.randn(dim, dstate)
        C_base = torch.randn(dim, dstate)

        # Method 1: loss.backward()
        u1 = u_base.clone().requires_grad_(True)
        delta1 = delta_base.clone().requires_grad_(True)
        A1 = A_base.clone().requires_grad_(True)
        B1 = B_base.clone().requires_grad_(True)
        C1 = C_base.clone().requires_grad_(True)

        out1 = selective_scan_fn(u1, delta1, A1, B1, C1)
        loss1 = out1.sum()
        loss1.backward()

        grad_u1 = u1.grad.clone()
        grad_delta1 = delta1.grad.clone()
        grad_A1 = A1.grad.clone()
        grad_B1 = B1.grad.clone()
        grad_C1 = C1.grad.clone()

        # Method 2: torch.autograd.grad()
        u2 = u_base.clone().requires_grad_(True)
        delta2 = delta_base.clone().requires_grad_(True)
        A2 = A_base.clone().requires_grad_(True)
        B2 = B_base.clone().requires_grad_(True)
        C2 = C_base.clone().requires_grad_(True)

        out2 = selective_scan_fn(u2, delta2, A2, B2, C2)
        loss2 = out2.sum()

        (grad_u2, grad_delta2, grad_A2, grad_B2, grad_C2) = torch.autograd.grad(
            loss2, [u2, delta2, A2, B2, C2], retain_graph=False
        )

        # Verify gradients match
        torch.testing.assert_close(grad_u1, grad_u2, atol=1e-6, rtol=1e-6)
        torch.testing.assert_close(grad_delta1, grad_delta2, atol=1e-6, rtol=1e-6)
        torch.testing.assert_close(grad_A1, grad_A2, atol=1e-6, rtol=1e-6)
        torch.testing.assert_close(grad_B1, grad_B2, atol=1e-6, rtol=1e-6)
        torch.testing.assert_close(grad_C1, grad_C2, atol=1e-6, rtol=1e-6)

        # Verify gradients are non-zero
        self.assertGreater(grad_u1.abs().mean().item(), 1e-7)
        self.assertGreater(grad_delta1.abs().mean().item(), 1e-7)
        self.assertGreater(grad_A1.abs().mean().item(), 1e-7)
        self.assertGreater(grad_B1.abs().mean().item(), 1e-7)
        self.assertGreater(grad_C1.abs().mean().item(), 1e-7)


def run_selective_scan_gradient_tests():
    """Run all gradient tests with verbose output."""
    print("🧪 Running selective scan gradient tests...")

    try:
        suite = unittest.TestLoader().loadTestsFromTestCase(TestSelectiveScanGradients)
        runner = unittest.TextTestRunner(verbosity=2)
        result = runner.run(suite)

        if result.wasSuccessful():
            print("✅ All selective scan gradient tests passed")
            return True
        else:
            print(f"❌ {len(result.failures)} failures, {len(result.errors)} errors")
            return False
    except Exception as e:
        print(f"❌ Test execution error: {e}")
        return False


if __name__ == "__main__":
    run_selective_scan_gradient_tests()



import json

import torch
from transformers.utils import CONFIG_NAME, WEIGHTS_NAME
from transformers.utils.hub import cached_file


def load_config_hf(model_name):
    resolved_archive_file = cached_file(
        model_name, CONFIG_NAME, _raise_exceptions_for_missing_entries=False
    )
    return json.load(open(resolved_archive_file))


def load_state_dict_hf(model_name, device=None, dtype=None):
    mapped_device = "cpu" if dtype not in [torch.float32, None] else device
    resolved_archive_file = cached_file(
        model_name, WEIGHTS_NAME, _raise_exceptions_for_missing_entries=False
    )
    state_dict = torch.load(resolved_archive_file, map_location=mapped_device, weights_only=True)
    if dtype is not None:
        state_dict = {k: v.to(dtype=dtype) for k, v in state_dict.items()}
    if device is not None:
        state_dict = {k: v.to(device=device) for k, v in state_dict.items()}
    return state_dict



#!/usr/bin/env python3
"""
Simple Training Example

Demonstrates how to train a simple language model using Mamba modules.
Shows forward pass, loss calculation, backward pass, and optimizer step.
Supports both Mamba 1 (SSM) and Mamba 2 (SSD) architectures.
"""

import random
from pathlib import Path
from typing import Iterable, Tuple

import numpy as np
import torch
import torch.nn as nn
import torch.optim as optim
from hydra.utils import instantiate
from omegaconf import DictConfig, OmegaConf
from torch.utils.data import DataLoader, Dataset
from tqdm import tqdm

from mamba_ssm.modules.mamba2 import Mamba2
from mamba_ssm.modules.mamba_simple import Mamba

DEVICE = "mps" if torch.backends.mps.is_available() else "cpu"
CONFIG = OmegaConf.load(Path(__file__).with_suffix(".yaml"))
LAYER_CONFIG = CONFIG.layer_config


def set_global_seed(seed: int) -> None:
    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    if torch.cuda.is_available():
        torch.cuda.manual_seed_all(seed)


class LearnableSequenceDataset(Dataset):
    def __init__(self, *, size: int, seq_len: int, vocab_size: int, task: str, base_seed: int):
        self._size, self._seq_len, self._vocab_size = size, seq_len, vocab_size
        self._task, self._base_seed = task, base_seed

    def __len__(self) -> int:
        return self._size

    def __getitem__(self, idx: int) -> Tuple[torch.Tensor, torch.Tensor]:
        generator = torch.Generator(device=DEVICE)
        generator.manual_seed(self._base_seed + idx)
        input_ids = torch.randint(
            0, self._vocab_size, (self._seq_len,), device=DEVICE, generator=generator
        )
        if self._task == "shift":
            labels = torch.roll(input_ids, shifts=1, dims=0)
            labels[0] = 0
        else:
            labels = input_ids.clone()
        return input_ids, labels


def create_dataloader(dataset: Dataset, batch_size: int) -> DataLoader:
    return DataLoader(dataset, batch_size=batch_size, shuffle=True, drop_last=True)


class SimpleMambaLM(nn.Module):
    def __init__(self, vocab_size: int, d_model: int, d_state: int, n_layers: int):
        super().__init__()
        self.embedding = nn.Embedding(vocab_size, d_model)
        self.layers = nn.ModuleList(
            [
                Mamba(
                    d_model=d_model,
                    d_state=d_state,
                    d_conv=LAYER_CONFIG["d_conv"],
                    expand=LAYER_CONFIG["expand"],
                    device=DEVICE,
                )
                for _ in range(n_layers)
            ]
        )
        self.norm = nn.LayerNorm(d_model)
        self.lm_head = nn.Linear(d_model, vocab_size, bias=False)
        self.to(DEVICE)

    def forward(self, input_ids):
        x = self.embedding(input_ids)
        for layer in self.layers:
            x = layer(x)
        x = self.norm(x)
        return self.lm_head(x)


class SimpleMamba2LM(nn.Module):
    def __init__(self, vocab_size: int, d_model: int, d_state: int, n_layers: int, headdim: int):
        super().__init__()
        self.embedding = nn.Embedding(vocab_size, d_model)
        self.layers = nn.ModuleList(
            [
                Mamba2(
                    d_model=d_model,
                    d_state=d_state,
                    d_conv=LAYER_CONFIG["d_conv"],
                    expand=LAYER_CONFIG["expand"],
                    headdim=headdim,
                    ngroups=LAYER_CONFIG["ngroups"],
                    chunk_size=LAYER_CONFIG["chunk_size"],
                    device=DEVICE,
                )
                for _ in range(n_layers)
            ]
        )
        self.norm = nn.LayerNorm(d_model)
        self.lm_head = nn.Linear(d_model, vocab_size, bias=False)
        self.to(DEVICE)

    def forward(self, input_ids):
        x = self.embedding(input_ids)
        for layer in self.layers:
            x = layer(x)
        x = self.norm(x)
        return self.lm_head(x)


def train_step(
    model: nn.Module,
    input_ids: torch.Tensor,
    labels: torch.Tensor,
    optimizer: optim.Optimizer,
    criterion: nn.Module,
) -> float:
    model.train()
    optimizer.zero_grad()
    logits = model(input_ids)
    loss = criterion(logits.view(-1, logits.size(-1)), labels.view(-1))
    loss.backward()
    optimizer.step()
    return loss.item()


def evaluate(model: nn.Module, dataloader: Iterable, criterion: nn.Module) -> Tuple[float, float]:
    model.eval()
    total_loss = 0.0
    total_correct = 0.0
    total_tokens = 0
    with torch.no_grad():
        for input_ids, labels in dataloader:
            logits = model(input_ids)
            loss = criterion(logits.view(-1, logits.size(-1)), labels.view(-1))
            predictions = logits.argmax(dim=-1)
            total_correct += (predictions == labels).float().sum().item()
            total_tokens += labels.numel()
            total_loss += loss.item()
    avg_loss = total_loss / max(1, len(dataloader))
    accuracy = total_correct / max(1, total_tokens)
    return avg_loss, accuracy


def train(model: nn.Module, model_name: str, cfg: DictConfig) -> float:
    print(f"Parameters: {sum(p.numel() for p in model.parameters()):,}")
    optimizer = optim.AdamW(model.parameters(), lr=cfg.lr)
    criterion = nn.CrossEntropyLoss()

    train_size = cfg.batch_size * cfg.num_steps
    train_dataset = LearnableSequenceDataset(
        size=train_size,
        seq_len=cfg.seq_len,
        vocab_size=cfg.vocab_size,
        task=cfg.task,
        base_seed=cfg.seed,
    )
    eval_dataset = LearnableSequenceDataset(
        size=cfg.batch_size * cfg.eval_batches,
        seq_len=cfg.seq_len,
        vocab_size=cfg.vocab_size,
        task=cfg.task,
        base_seed=cfg.seed + train_size,
    )
    train_loader = create_dataloader(train_dataset, batch_size=cfg.batch_size)
    eval_loader = create_dataloader(eval_dataset, batch_size=cfg.batch_size)

    last_accuracy = 0.0
    for step, (input_ids, labels) in tqdm(
        enumerate(train_loader, start=1),
        desc=f"Training {model_name.upper()}",
        total=cfg.num_steps,
    ):
        loss = train_step(model, input_ids, labels, optimizer, criterion)
        if step % cfg.eval_every_steps == 0 or step == 1:
            test_loss, accuracy = evaluate(model, eval_loader, criterion)
            last_accuracy = accuracy
            tqdm.write(
                f"Step {step}, Loss: {loss:.4f}, "
                f"Test Loss: {test_loss:.4f}, Accuracy: {accuracy:.2%}"
            )
    if last_accuracy == 0.0:
        _, last_accuracy = evaluate(model, eval_loader, criterion)
    return last_accuracy


def print_report(cfg: DictConfig, results: dict) -> None:
    print("\n📊 Training Report")
    print("=" * 50)
    print(f"Task: {cfg.task} | Steps: {cfg.num_steps} | Batch size: {cfg.batch_size}")
    print(
        "Dataset sizes: "
        f"train={cfg.batch_size}×{cfg.num_steps}, "
        f"eval={cfg.batch_size}×{cfg.eval_batches}"
    )
    print(f"\nFinal test accuracy after training for {cfg.num_steps} steps:")
    for name, accuracy in results.items():
        title = CONFIG.models[name].title
        print(f"- {title}: {accuracy:.2%}")
    print("\n💡 Both architectures support full training with backpropagation!")


def main() -> None:
    print("🎓 Simple Training Example")
    print("=" * 50)
    print(f"Device: {DEVICE}")
    set_global_seed(CONFIG.training.seed)

    print(f"Task: {CONFIG.training.task}")
    print(f"Training steps: {CONFIG.training.num_steps}")

    results = {}
    for name, config in CONFIG.models.items():
        print(f"\n--- Training {config.title} ---")
        model = instantiate(config.model)
        results[name] = train(model=model, model_name=name, cfg=CONFIG.training)

    print_report(CONFIG.training, results)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import sys
from pathlib import Path

sys.path.append(str(Path(__file__).parent.parent))

from tests.unit.test_generation_macos import run_generation_tests
from tests.unit.test_mamba2_macos import run_mamba2_tests
from tests.unit.test_mamba_macos import run_mamba_tests
from tests.unit.test_selective_scan_gradients import run_selective_scan_gradient_tests


def run_unit_tests():
    print("🚀 Starting unit test suite...")

    test_modules = [
        ("Mamba macOS Tests", run_mamba_tests),
        ("Mamba2 macOS Tests", run_mamba2_tests),
        ("Generation macOS Tests", run_generation_tests),
        ("Selective Scan Gradient Tests", run_selective_scan_gradient_tests),
    ]

    results = {}

    for test_name, test_func in test_modules:
        print(f"\n{'='*50}")
        print(f"Running {test_name}")
        print(f"{'='*50}")

        try:
            success = test_func()
            results[test_name] = success
            status = "✅ PASSED" if success else "❌ FAILED"
            print(f"{test_name}: {status}")

        except Exception as e:
            print(f"❌ {test_name} execution error: {e}")
            results[test_name] = False

    print(f"\n{'='*50}")
    print("TEST SUMMARY")
    print(f"{'='*50}")

    passed = sum(1 for success in results.values() if success)
    total = len(results)

    for test_name, success in results.items():
        status = "✅ PASSED" if success else "❌ FAILED"
        print(f"{test_name}: {status}")

    print(f"\nOverall: {passed}/{total} test modules passed")

    if passed == total:
        print("🎉 All unit tests passed!")
        return True
    else:
        print("❌ Some unit tests failed")
        return False


def main():
    success = run_unit_tests()
    sys.exit(0 if success else 1)


if __name__ == "__main__":
    main()



from typing import Optional

import torch
from torch import Tensor, nn


class Block(nn.Module):
    def __init__(self, dim, mixer_cls, mlp_cls, norm_cls=nn.LayerNorm, residual_in_fp32=False):
        super().__init__()
        self.residual_in_fp32 = residual_in_fp32
        self.norm = norm_cls(dim)
        self.mixer = mixer_cls(dim)
        if mlp_cls is not nn.Identity:
            self.norm2 = norm_cls(dim)
            self.mlp = mlp_cls(dim)
        else:
            self.mlp = None

    def forward(
        self,
        hidden_states: Tensor,
        residual: Optional[Tensor] = None,
        inference_params=None,
        **mixer_kwargs,
    ):
        residual = (hidden_states + residual) if residual is not None else hidden_states
        hidden_states = self.norm(residual.to(dtype=self.norm.weight.dtype))
        if self.residual_in_fp32:
            residual = residual.to(torch.float32)
        hidden_states = self.mixer(hidden_states, inference_params=inference_params, **mixer_kwargs)
        if self.mlp is not None:
            residual = hidden_states + residual
            hidden_states = self.norm2(residual.to(dtype=self.norm2.weight.dtype))
            if self.residual_in_fp32:
                residual = residual.to(torch.float32)
            hidden_states = self.mlp(hidden_states)
        return hidden_states, residual

    def allocate_inference_cache(self, batch_size, max_seqlen, dtype=None, **kwargs):
        return self.mixer.allocate_inference_cache(batch_size, max_seqlen, dtype=dtype, **kwargs)



#!/usr/bin/env python3
"""
Basic Core Modules Usage Example

Demonstrates fundamental usage patterns for core Mamba modules on Apple Silicon.
Shows both Mamba 1 (Mamba) and Mamba 2 (Mamba2) architectures.
"""

import torch

from mamba_ssm.modules.mamba2 import Mamba2
from mamba_ssm.modules.mamba_simple import Mamba


def demo_mamba1(device):
    print("\n🔷 Mamba 1 (SSM Architecture)")
    print("=" * 40)
    block = Mamba(
        d_model=512,
        d_state=16,
        d_conv=4,
        expand=2,
        dt_min=0.001,
        dt_max=0.1,
        device=device,
    )
    print(f"Parameters: {sum(p.numel() for p in block.parameters()):,}")
    input_features = torch.randn(8, 1024, 512, device=device)
    with torch.no_grad():
        output = block(input_features)
    print(f"Input shape:  {input_features.shape}")
    print(f"Output shape: {output.shape}")
    print(f"Sample output: {output[0, 0, :5]}")
    print("✅ Mamba 1 forward pass successful!")


def demo_mamba2(device):
    print("\n🔶 Mamba 2 (SSD Architecture)")
    print("=" * 40)
    block = Mamba2(
        d_model=512,
        d_state=64,
        d_conv=4,
        expand=2,
        headdim=128,
        ngroups=1,
        chunk_size=256,
        device=device,
    )
    print(f"Parameters: {sum(p.numel() for p in block.parameters()):,}")
    input_features = torch.randn(8, 1024, 512, device=device)
    with torch.no_grad():
        output = block(input_features)
    print(f"Input shape:  {input_features.shape}")
    print(f"Output shape: {output.shape}")
    print(f"Sample output: {output[0, 0, :5]}")
    print("✅ Mamba 2 forward pass successful!")
    print("\n🔄 Step function demo:")
    conv_state, ssm_state = block.allocate_inference_cache(8, 1024)
    x_step = input_features[:, :1, :]
    with torch.no_grad():
        y_step, new_conv_state, new_ssm_state = block.step(x_step, conv_state, ssm_state)
    print(f"Step input:  {x_step.shape}")
    print(f"Step output: {y_step.shape}")
    print("✅ Step function successful!")


def main():
    print("🍎 Basic Core Modules Usage")
    print("=" * 50)

    device = "mps" if torch.backends.mps.is_available() else "cpu"
    print(f"Device: {device}")

    demo_mamba1(device)
    demo_mamba2(device)

    print("\n📊 Summary")
    print("=" * 50)
    print("✅ Mamba 1: SSM architecture, d_state=16")
    print("✅ Mamba 2: SSD architecture, d_state=64, supports step()")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import argparse
import json
import random
import sys
import tempfile
from pathlib import Path

import numpy as np
import torch

sys.path.append(str(Path(__file__).parent.parent.parent / "scripts"))

from download_models import download_model

from mamba_ssm import generate_text_with_model, get_device, load_and_prepare_model


def set_seed(s=42):
    random.seed(s)
    np.random.seed(s)
    torch.manual_seed(s)
    if torch.cuda.is_available():
        torch.cuda.manual_seed_all(s)
    if torch.backends.mps.is_available():
        torch.mps.manual_seed(s)
    torch.backends.cudnn.deterministic, torch.backends.cudnn.benchmark = True, False


def test_device_selection():
    device = get_device()
    print(f"Selected device: {device}")

    if device == "mps":
        assert torch.backends.mps.is_available() and torch.backends.mps.is_built()
        print("✅ MPS acceleration available")
    elif device == "cuda":
        assert torch.cuda.is_available()
        print("✅ CUDA acceleration available")
    else:
        print("✅ Using CPU (no acceleration)")

    return device


def test_download_functionality(temp_dir):
    print("\n🔽 Testing download functionality...")

    for model_type in ["mamba1", "mamba2"]:
        print(f"📦 Testing {model_type} download...")
        try:
            download_model(model_type, temp_dir)

            expected_files = [
                f"{model_type}-130m-config.json",
                f"{model_type}-130m-model.bin",
            ]
            model_dir = Path(temp_dir) / model_type

            for file_name in expected_files:
                file_path = model_dir / file_name
                assert file_path.exists(), f"{file_name} not found"
                print(f"   ✅ {file_name} downloaded")

            config_path = model_dir / f"{model_type}-130m-config.json"
            with open(config_path) as f:
                config = json.load(f)
            assert "d_model" in config and "ssm_cfg" in config
            print(f"   ✅ {model_type} config validated")

        except Exception as e:
            print(f"   ❌ {model_type} download failed: {e}")
            return False

    print("✅ All downloads successful")
    return True


def test_model_loading(model_dir, device):
    print("\n🏗️ Testing model loading...")

    for model_type in ["mamba1", "mamba2"]:
        print(f"🔄 Loading {model_type}...")
        try:
            success, model, tokenizer = load_and_prepare_model(model_type, model_dir, device)

            if not success:
                print(f"   ❌ {model_type} loading failed")
                return False

            assert model is not None and tokenizer is not None and hasattr(model, "forward")

            total_params = sum(p.numel() for p in model.parameters())
            print(f"   ✅ {model_type} loaded: {total_params:,} parameters")

        except Exception as e:
            print(f"   ❌ {model_type} loading error: {e}")
            return False

    print("✅ All models loaded successfully")
    return True


def test_text_generation(model_dir, device):
    print("\n📝 Testing text generation...")
    set_seed()
    test_prompts = ["The weather today is", "AI technology", "Machine learning"]
    for model_type in ["mamba1", "mamba2"]:
        print(f"🤖 Testing {model_type} generation...")
        try:
            success, model, tokenizer = load_and_prepare_model(model_type, model_dir, device)
            if not success:
                print(f"   ❌ Failed to load {model_type}")
                return False
            for prompt in test_prompts:
                set_seed()
                generated = generate_text_with_model(
                    model, tokenizer, prompt, device, max_length=30, temperature=0.7
                )
                assert generated and len(generated) > len(prompt) and isinstance(generated, str)
                print(f"   📝 Input: '{prompt}'\n   ✅ Output: '{generated}'")
        except Exception as e:
            print(f"   ❌ {model_type} generation error: {e}")
            return False
    print("✅ All generation tests passed")
    return True


def test_parameter_variations(model_dir, device):
    print("\n🎛️ Testing parameter variations...")
    set_seed()
    success, model, tokenizer = load_and_prepare_model("mamba1", model_dir, device)
    if not success:
        print("❌ Failed to load model for parameter testing")
        return False
    prompt = "Test generation"
    try:
        print(f"📝 Input: '{prompt}'")
        set_seed()
        short = generate_text_with_model(
            model, tokenizer, prompt, device, max_length=20, temperature=0.5
        )
        set_seed()
        long = generate_text_with_model(
            model, tokenizer, prompt, device, max_length=50, temperature=0.5
        )
        set_seed()
        cold = generate_text_with_model(
            model, tokenizer, prompt, device, max_length=30, temperature=0.1
        )
        set_seed()
        hot = generate_text_with_model(
            model, tokenizer, prompt, device, max_length=30, temperature=1.0
        )
        print(
            f"   🔵 Short (max=20): '{short}'\n   🟡 Long (max=50): '{long}'\n   ❄️  Cold (temp=0.1): '{cold}'\n   🔥 Hot (temp=1.0): '{hot}'"
        )
        assert len(short) <= len(long) and all(isinstance(t, str) for t in [short, long, cold, hot])
        print("✅ Length and temperature parameters working")
    except Exception as e:
        print(f"❌ Parameter testing error: {e}")
        return False
    print("✅ Parameter variation tests passed")
    return True


def run_comprehensive_test():
    print("🧪 Starting comprehensive unified system test...")
    set_seed()
    device = test_device_selection()
    with tempfile.TemporaryDirectory() as temp_dir:
        print(f"📁 Using temporary directory: {temp_dir}")
        tests = [
            (test_download_functionality, (temp_dir,), "Download tests"),
            (test_model_loading, (temp_dir, device), "Model loading tests"),
            (test_text_generation, (temp_dir, device), "Text generation tests"),
            (test_parameter_variations, (temp_dir, device), "Parameter variation tests"),
        ]
        for f, args, name in tests:
            if not f(*args):
                print(f"❌ {name} failed")
                return False
    print("\n🎉 ALL TESTS PASSED!\n✅ Unified system is working perfectly")
    return True


def run_quick_test(model_dir):
    print("⚡ Running quick test with existing models...")
    set_seed()
    device = get_device()
    for model_type in ["mamba1", "mamba2"]:
        print(f"\n🤖 Testing {model_type}...")
        success, model, tokenizer = load_and_prepare_model(model_type, model_dir, device)
        if success:
            prompt = "The future of AI"
            set_seed()
            generated = generate_text_with_model(
                model, tokenizer, prompt, device, max_length=20, temperature=0.7
            )
            print(f"📝 Input: '{prompt}'\n✅ Output: '{generated}'")
        else:
            print(f"❌ {model_type} not available")
    print("⚡ Quick test completed")


def main():
    parser = argparse.ArgumentParser(description="Test unified Mamba system")
    parser.add_argument(
        "--comprehensive",
        action="store_true",
        help="Run comprehensive test with downloads",
    )
    parser.add_argument(
        "--model-dir",
        default="./models",
        help="Existing model directory for quick test",
    )

    args = parser.parse_args()

    if args.comprehensive:
        success = run_comprehensive_test()
        print("✅ SUCCESS" if success else "❌ FAILED")
    else:
        run_quick_test(args.model_dir)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import argparse
import json
from pathlib import Path

from huggingface_hub import hf_hub_download, snapshot_download
from tqdm import tqdm


def create_config(source_config_path, output_config_path, model_type):
    config_data = json.load(open(source_config_path))

    if model_type == "mamba1":
        if "ssm_cfg" not in config_data:
            config_data["ssm_cfg"] = {}
        if "layer" not in config_data["ssm_cfg"]:
            config_data["ssm_cfg"]["layer"] = "Mamba1"
    else:
        config_data["ssm_cfg"] = {
            "layer": "Mamba2",
            "d_state": 128,
            "d_conv": 4,
            "expand": 2,
            "headdim": 64,
            "ngroups": 1,
        }

    config_data["fused_add_norm"] = False
    with open(output_config_path, "w") as f:
        json.dump(config_data, f, indent=2)
    return output_config_path


def download_model(model_type, model_dir):
    local_dir = f"{model_dir}/{model_type}"
    print(f"Downloading state-spaces/{model_type}-130m to {local_dir}")
    Path(local_dir).mkdir(parents=True, exist_ok=True)

    if model_type == "mamba1":
        print("📦 Downloading config.json...")
        config_path = hf_hub_download(
            "state-spaces/mamba-130m",
            "config.json",
            local_dir=local_dir,
            resume_download=True,
        )
        print("📦 Downloading pytorch_model.bin...")
        model_path = hf_hub_download(
            "state-spaces/mamba-130m",
            "pytorch_model.bin",
            local_dir=local_dir,
            resume_download=True,
        )

        print("🔄 Renaming files...")
        final_config = f"{local_dir}/{model_type}-130m-config.json"
        final_model = f"{local_dir}/{model_type}-130m-model.bin"
        Path(config_path).rename(final_config)
        Path(model_path).rename(final_model)
        downloaded_files = [
            f"{model_type}-130m-config.json",
            f"{model_type}-130m-model.bin",
        ]
    else:
        print("📦 Downloading all files...")
        snapshot_download(
            "state-spaces/mamba2-130m",
            local_dir=local_dir,
            resume_download=True,
            local_dir_use_symlinks=False,
            tqdm_class=tqdm,
        )

        print("🔄 Renaming files...")
        original_config = f"{local_dir}/config.json"
        final_config = f"{local_dir}/{model_type}-130m-config.json"
        final_model = f"{local_dir}/{model_type}-130m-model.bin"

        if Path(original_config).exists():
            create_config(original_config, final_config, model_type)
        Path(f"{local_dir}/pytorch_model.bin").rename(final_model)
        downloaded_files = [
            f"{model_type}-130m-config.json",
            f"{model_type}-130m-model.bin",
        ]

    print("✅ Download complete")
    print(f"📁 Files saved to: {local_dir}/")
    for file_name in downloaded_files:
        print(f"   - {file_name}")


def main():
    parser = argparse.ArgumentParser(description="Download Mamba models")
    parser.add_argument("model", choices=["mamba1", "mamba2", "both"], help="Model to download")
    parser.add_argument("--model-dir", default="./models", help="Downloaded models directory")
    args = parser.parse_args()

    try:
        if args.model in ["mamba1", "both"]:
            download_model("mamba1", args.model_dir)
        if args.model in ["mamba2", "both"]:
            download_model("mamba2", args.model_dir)
        return True
    except Exception as e:
        print(f"❌ Error: {e}")
        return False


if __name__ == "__main__":
    print("✅ SUCCESS" if main() else "❌ FAILED")



#!/usr/bin/env python3
"""
Mamba Demo - Pretrained Model Text Generation

This example demonstrates text generation using downloaded Mamba1 and Mamba2 models.

Features:
- Uses pretrained Mamba1 and Mamba2 models (both with 130M parameters)
- Supports both architectures
- Interactive and automated generation modes
- Shows proper device selection and error handling
"""

import argparse
import time

from mamba_ssm import generate_text_with_model, get_device, load_and_prepare_model


def demo_model(model_name, prompts, model_dir="./models"):
    """Demonstrate a specific model with various prompts."""
    print(f"\n🤖 Testing {model_name.upper()}")
    print("=" * 50)

    device = get_device()
    print(f"📱 Device: {device}")

    # Load model
    success, model, tokenizer = load_and_prepare_model(model_name, model_dir, device)

    if not success:
        print(f"❌ Failed to load {model_name}")
        return False

    # Test with different prompts
    for i, prompt_config in enumerate(prompts, 1):
        prompt = prompt_config["prompt"]
        max_length = prompt_config.get("max_length", 40)
        temperature = prompt_config.get("temperature", 0.7)

        print(f"\n📝 Test {i}: '{prompt}'")
        print(f"⚙️ Settings: max_length={max_length}, temperature={temperature}")

        start_time = time.time()
        try:
            generated = generate_text_with_model(
                model, tokenizer, prompt, device, max_length, temperature
            )
            gen_time = time.time() - start_time

            print(f"✅ Generated ({gen_time:.2f}s):")
            print(f"   {generated}")

            # Calculate words per second
            words_generated = len(generated.split()) - len(prompt.split())
            if words_generated > 0 and gen_time > 0:
                wps = words_generated / gen_time
                print(f"📊 Performance: {wps:.1f} words/sec")

        except Exception as e:
            print(f"❌ Generation failed: {e}")

    return True


def interactive_mode():
    """Interactive mode for custom prompts."""
    print("\n🎯 Interactive Mode")
    print("=" * 50)
    print("Enter prompts to test with both models (type 'quit' to exit)")

    device = get_device()
    models = {}

    # Pre-load both models
    for model_name in ["mamba1", "mamba2"]:
        print(f"\n🔄 Loading {model_name}...")
        success, model, tokenizer = load_and_prepare_model(model_name, "./models", device)

        if success:
            models[model_name] = (model, tokenizer)
            print(f"✅ {model_name} ready")
        else:
            print(f"❌ {model_name} failed to load")

    if not models:
        print("❌ No models available for interactive mode")
        return

    while True:
        try:
            prompt = input("\n💭 Enter your prompt: ").strip()

            if prompt.lower() in ["quit", "exit", "q"]:
                print("👋 Goodbye!")
                break

            if not prompt:
                continue

            # Get parameters
            try:
                max_length = int(input("📏 Max length (default 30): ") or "30")
                temperature = float(input("🌡️ Temperature (default 0.7): ") or "0.7")
            except ValueError:
                max_length, temperature = 30, 0.7

            # Test with all available models
            for model_name, (model, tokenizer) in models.items():
                print(f"\n🤖 {model_name.upper()}:")
                try:
                    generated = generate_text_with_model(
                        model, tokenizer, prompt, device, max_length, temperature
                    )
                    print(f"   {generated}")
                except Exception as e:
                    print(f"   ❌ Error: {e}")

        except KeyboardInterrupt:
            print("\n👋 Interrupted by user. Goodbye!")
            break


def showcase_structure():
    """Show the organized directory structure."""
    print("\n🏗️ Organized Directory Structure")
    print("=" * 50)

    structure_info = [
        ("📦 mamba_ssm/", "Core library"),
        ("🔧 scripts/", "Utility scripts (download, run models)"),
        ("🧪 tests/", "Unit and integration tests"),
        ("📚 examples/", "Usage examples and demos"),
        ("🤖 models/", "Downloaded model files"),
    ]

    for path, description in structure_info:
        print(f"{path:<25} {description}")

    print(f"\n🎯 Key Benefits:")
    print("✅ Clean separation of concerns")
    print("✅ Easy imports: from mamba_ssm import ...")
    print("✅ Organized tests (unit + integration)")
    print("✅ Makefile for common development tasks")


def main():
    parser = argparse.ArgumentParser(description="Mamba pretrained Demo")
    parser.add_argument("--interactive", action="store_true", help="Interactive mode")
    parser.add_argument(
        "--model",
        choices=["mamba1", "mamba2", "both"],
        default="both",
        help="Model to test",
    )
    parser.add_argument("--show-structure", action="store_true", help="Show directory structure")

    args = parser.parse_args()

    print("🎉 Mamba inference demo")
    print("=" * 50)
    print("Text generation with pretrained Mamba models")

    if args.show_structure:
        showcase_structure()
        return

    if args.interactive:
        interactive_mode()
        return

    # Predefined demo prompts
    demo_prompts = [
        {
            "prompt": "The future of artificial intelligence",
            "max_length": 35,
            "temperature": 0.7,
        },
        {
            "prompt": "Apple Silicon processors are revolutionizing",
            "max_length": 30,
            "temperature": 0.8,
        },
        {
            "prompt": "State space models represent",
            "max_length": 25,
            "temperature": 0.6,
        },
    ]

    models_to_test = []
    if args.model in ["mamba1", "both"]:
        models_to_test.append("mamba1")
    if args.model in ["mamba2", "both"]:
        models_to_test.append("mamba2")

    success_count = 0
    for model_name in models_to_test:
        if demo_model(model_name, demo_prompts):
            success_count += 1

    print(f"\n🎯 Demo Summary")
    print("=" * 50)
    print(f"✅ Successfully tested {success_count}/{len(models_to_test)} models")
    print("📋 Try these commands:")
    print("  python -m examples.02_text_generation --interactive")
    print("  python -m examples.02_text_generation --show-structure")
    print("  python -m examples.02_text_generation --model mamba1")
    print("\n🛠️ Development commands:")
    print("  make test-quick    # Quick integration test")
    print("  make run-mamba1    # Run Mamba1 demo")
    print("  make show-structure # Show project layout")


if __name__ == "__main__":
    main()



import time
from dataclasses import dataclass, field
from typing import Callable, Optional

import torch
from torch import Tensor
from transformers.generation import GenerateDecoderOnlyOutput, TextStreamer


@dataclass
class InferenceParams:
    """Inference parameters that are passed to the main model in order
    to efficienly calculate and store the context during inference."""

    max_seqlen: int
    max_batch_size: int
    seqlen_offset: int = 0
    batch_size_offset: int = 0
    key_value_memory_dict: dict = field(default_factory=dict)
    lengths_per_sample: Optional[Tensor] = None

    def reset(self, max_seqlen, max_batch_size):
        self.max_seqlen = max_seqlen
        self.max_batch_size = max_batch_size
        self.seqlen_offset = 0
        if self.lengths_per_sample is not None:
            self.lengths_per_sample.zero_()


def modify_logits_for_min_p_filtering(logits, min_p):
    if min_p <= 0.0 or min_p >= 1.0:
        return
    logits.masked_fill_(logits < min_p, float("-Inf"))


def modify_logits_for_top_p_filtering(logits, top_p):
    """Set the logits for none top-p values to -inf. Done in-place."""
    if top_p <= 0.0 or top_p >= 1.0:
        return
    # First sort and calculate cumulative sum of probabilities.
    sorted_logits, sorted_indices = torch.sort(logits, descending=False)
    cumulative_probs = sorted_logits.softmax(dim=-1).cumsum(dim=-1)
    # Remove tokens with cumulative top_p above the threshold (token with 0 are kept)
    sorted_indices_to_remove = cumulative_probs <= (1 - top_p)
    # scatter sorted tensors to original indexing
    indices_to_remove = sorted_indices_to_remove.scatter(
        1, sorted_indices, sorted_indices_to_remove
    )
    logits.masked_fill_(indices_to_remove, float("-inf"))


def modify_logit_for_repetition_penalty(logits, prev_output_tokens, repetition_penalty=1.0):
    """Apply repetition penalty. See https://arxiv.org/abs/1909.05858
    logits: (batch_size, vocab_size)
    prev_output_tokens: (batch_size, seq_len)
    """
    if repetition_penalty == 1.0:
        return logits
    score = torch.gather(logits, 1, prev_output_tokens)
    # if score < 0 then repetition penalty has to be multiplied to reduce the previous token probability
    score = torch.where(score < 0, score * repetition_penalty, score / repetition_penalty)
    logits.scatter_(1, prev_output_tokens, score)
    return logits


def sample(logits, top_k=1, top_p=0.0, min_p=0.0, temperature=1.0):
    """Sample from top-k logits.
    Arguments:
        logits: Tensor of shape (batch_size, vocab_size)
    """
    if top_k == 1:  # Short-circuit for greedy decoding
        return logits.argmax(dim=-1)
    else:
        if top_p > 0.0:
            assert top_p <= 1.0, "top-p should be in (0, 1]."
        if top_k > 0:
            top_k = min(top_k, logits.size(-1))  # Safety check
            logits_top, indices = torch.topk(logits, top_k, dim=-1)
            if temperature != 1.0:
                logits_top /= temperature
            modify_logits_for_top_p_filtering(logits_top, top_p)
            return indices[
                torch.arange(indices.shape[0], device=indices.device),
                torch.multinomial(torch.softmax(logits_top, dim=-1), num_samples=1).squeeze(dim=-1),
            ]
        else:
            if min_p > 0.0:
                logits_top = logits.clone()
                max_prob = logits_top[..., 0].item()
                min_prob = max_prob * min_p
                modify_logits_for_min_p_filtering(logits_top, min_prob)
                if temperature != 1.0:
                    logits_top /= temperature
                return torch.multinomial(torch.softmax(logits_top, dim=-1), num_samples=1).squeeze(
                    dim=-1
                )
            # Clone so that when we modify for top_p we don't change the original logits
            logits_top = logits / temperature if temperature != 1.0 else logits.clone()
            modify_logits_for_top_p_filtering(logits_top, top_p)
            return torch.multinomial(torch.softmax(logits_top, dim=-1), num_samples=1).squeeze(
                dim=-1
            )


@torch.inference_mode()
def decode(
    input_ids,
    model,
    max_length,
    top_k=1,
    top_p=0.0,
    min_p=0.0,
    temperature=1.0,
    repetition_penalty=1.0,
    eos_token_id=None,
    teacher_outputs=None,
    vocab_size=None,
    cg=False,
    enable_timing=False,
    output_scores=False,
    streamer: Optional[TextStreamer] = None,
):
    """Decoding, either greedy or with top-k or top-p sampling.
    If top-k = 0, don't limit the number of candidates (pure sampling).
    Top-k and top-p can be used together. If top_k > 0 and top_p > 0, then top-k is applied first,
    then top-p.
    We assume that all sequences in the same batch have the same length.

    Arguments:
        input_ids: (batch, seq_len)
        max_length: int
        teacher_outputs (optional): (batch, seq_len). If provided, instead of sampling from the
            logits, the next token is taken from the teacher_outputs. Useful for testing.
    Returns: GenerateDecoderOnlyOutput with the following fields:
        sequences: (batch, max_length)
        scores: tuples of (batch, vocab_size)
    """
    if streamer is not None:
        streamer.put(input_ids.cpu())

    batch_size, seqlen_og = input_ids.shape
    teacher_output_len = teacher_outputs.shape[1] if teacher_outputs is not None else 0
    if cg:
        if not hasattr(model, "_decoding_cache"):
            model._decoding_cache = None
        model._decoding_cache = update_graph_cache(
            model,
            model._decoding_cache,
            batch_size,
            seqlen_og,
            max_length,
        )
        inference_params = model._decoding_cache.inference_params
        inference_params.reset(max_length, batch_size)
    else:
        inference_params = InferenceParams(max_seqlen=max_length, max_batch_size=batch_size)

    def get_logits(input_ids, inference_params):
        decoding = inference_params.seqlen_offset > 0
        if decoding:
            position_ids = torch.full(
                (batch_size, 1),
                inference_params.seqlen_offset,
                dtype=torch.long,
                device=input_ids.device,
            )
        else:
            position_ids = None
        if not cg or not decoding:
            logits = model(
                input_ids,
                position_ids=position_ids,
                inference_params=inference_params,
                num_last_tokens=1,
            ).logits.squeeze(dim=1)
        else:
            logits = model._decoding_cache.run(
                input_ids, position_ids, inference_params.seqlen_offset
            ).squeeze(dim=1)
        return logits[..., :vocab_size] if vocab_size is not None else logits

    def sample_tokens(logits, inference_params):
        if teacher_outputs is None or teacher_output_len <= inference_params.seqlen_offset:
            token = sample(logits, top_k=top_k, top_p=top_p, min_p=min_p, temperature=temperature)
        else:
            token = teacher_outputs[:, inference_params.seqlen_offset]
        # return rearrange(token, "b -> b 1")
        return token.unsqueeze(1)

    def should_stop(current_token, inference_params):
        if inference_params.seqlen_offset == 0:
            return False
        if eos_token_id is not None and (current_token == eos_token_id).all():
            return True
        if inference_params.seqlen_offset >= max_length - 1:
            return True
        return False

    start_time = time.time() if enable_timing else None
    scores, sequences = [], [input_ids]
    sequences_cat = input_ids
    while not should_stop(sequences[-1], inference_params):
        logits = get_logits(sequences[-1], inference_params)
        if output_scores:
            scores.append(logits.clone())
        inference_params.seqlen_offset += sequences[-1].shape[1]
        if repetition_penalty == 1.0:
            sampled_tokens = sample_tokens(logits, inference_params)
        else:
            logits = modify_logit_for_repetition_penalty(logits, sequences_cat, repetition_penalty)
            sampled_tokens = sample_tokens(logits, inference_params)
            sequences_cat = torch.cat([sequences_cat, sampled_tokens], dim=1)
        sequences.append(sampled_tokens)
        if streamer is not None:
            streamer.put(sampled_tokens.cpu())
    if streamer is not None:
        streamer.end()
    if enable_timing and start_time is not None:
        elapsed_ms = (time.time() - start_time) * 1000
        print(f"Prompt processing + decoding time: {elapsed_ms:.0f}ms")
    return GenerateDecoderOnlyOutput(sequences=torch.cat(sequences, dim=1), scores=tuple(scores))


class GenerationMixin:
    def allocate_inference_cache(self, batch_size, max_seqlen, dtype=None, **kwargs):
        raise NotImplementedError

    def generate(
        self,
        input_ids,
        max_length,
        top_k=1,
        top_p=0.0,
        min_p=0.0,
        temperature=1.0,
        return_dict_in_generate=False,
        output_scores=False,
        **kwargs,
    ):
        output = decode(
            input_ids,
            self,
            max_length,
            top_k=top_k,
            top_p=top_p,
            min_p=min_p,
            temperature=temperature,
            output_scores=output_scores,
            **kwargs,
        )
        if not output_scores:
            output.scores = None
        return output if return_dict_in_generate else output.sequences


@dataclass
class DecodingCGCache:
    max_batch_size: int = 0
    max_seqlen: int = 0
    device = None
    dtype = None
    callables: dict = field(default_factory=dict)
    mempool = None
    inference_params: Optional[InferenceParams] = None
    run: Optional[Callable] = None


@torch.inference_mode()
def update_graph_cache(
    model, cache, batch_size, seqlen_og, max_seqlen, decoding_seqlens=(1,), dtype=None, n_warmups=2
):
    raise NotImplementedError(
        "CUDA graph caching is not supported on macOS. Set cg=False in decode()."
    )


def capture_graph(
    model, inference_params, batch_size, max_seqlen, decoding_seqlen=1, mempool=None, n_warmups=2
):
    raise NotImplementedError(
        "CUDA graph capture is not supported on macOS. Set cg=False in decode()."
    )
